#ifndef MONITORING_H
#define MONITORING_H

#include "main.h"
#include "processing.h"
#include <arm_math.h>
#include "globals.h"

#define HEALTH_REF_FFT_LEN 512 // will need 4x the length -> 2 kB.

#define HEALTH_REF_SAMP_LEN (HEALTH_REF_FFT_LEN*2) // need 2n samples for n bins

#define HEALTH_REF_SAMPLE_RATE 5120

#define REF_MAGIC 0x5f3c3596

#define RPT_COST_X 10
#define RPT_COST_Y 5

typedef struct __attribute__((aligned(4)))
{
	float rms;
	uint32_t magic;
} reference_info;

/**
 * @brief Store captured vector as reference.
 *
 * Must be HEALTH_REF_SAMP_LEN long,
 * in the global buffer as uint32's
 *
 * @return success
 */
bool rpt_store_reference(void);

/**
 * @brief Process captured vector (for comparison / storing reference)
 *
 * works with samp_buf.
 *
 * @param nfo : info ptr
 */
void rpt_process_captured_vector(reference_info *nfo);

/**
 * @brief Calculate distance of the current FFT vector to the reference
 * @return distance
 */
float rpt_calc_vector_dist(void);

/** Check if reference is stored */
bool rpt_is_ref_stored(void);

#endif // MONITORING_H
